@@ -2,9 +2,15 @@ import typer
 from typer import Option
 from typing import Optional
 from thingsmatrix import apis
+from thingsmatrix import __version__
 from rich.console import Console
 
-app = typer.Typer()
+app_help = \
+f'''
+ThingsMatrix API TOOL
+version: {__version__}
+'''
+app = typer.Typer(help=app_help)
 
 console = Console()
 
@@ -18,13 +24,13 @@ def reports(sn: Optional[str] = typer.Argument(..., help='imei'),
             endtime: Optional[str] = typer.Option(
                 None, help='Format as YYYY-mm-dd HH:mm:ss')):
     '''
-    Filter reports by imei and datetime
+    Get reports by imei and datetime
     '''
     response, reports = api.get_devices_reports(sn=sn,
                                                 startTime=starttime,
                                                 endTime=endtime)
     if response != None or reports != None:
-        console.print(response.json()['data']['content'])
+        console.print(reports.json)
 
 
 # @app.command()
@@ -83,7 +89,7 @@ def device(sn: str,
                help="show gps report interval",
            )):
     '''
-    Get device by serial number
+    Get device info by imei
     '''
     device = api.get_device(sn)
     if device:
@@ -104,15 +110,29 @@ def device(sn: str,
         elif gps_report_interval:
             console.print(device.template.gps_report_interval)
         else:
-            device.printJson()
+            console.print(device.json)
 
 
+@app.command()
+def events(sn: str,
+           starttime: Optional[str] = typer.Option(
+               None, help='Format as YYYY-mm-dd HH:mm:ss'),
+           endtime: Optional[str] = typer.Option(
+               None, help='Format as YYYY-mm-dd HH:mm:ss')):
+    """Get events by imei and datetime
+    """
+    response, events = api.get_devices_events(sn, starttime, endtime)
+    if response != None or events != None:
+        console.print(events.json)
+
 def run():
     try:
         app()
         typer.Exit(0)
+    except ConnectionError as e:
+        console.log(e, log_locals=True)
     except Exception as e:
-        print(e)
+        console.print_exception(show_locals=True)
         typer.Exit(-1)
 
 
