@@ -8,6 +8,8 @@ from rich.text import Text
 from enum import Enum
 from .gredients import *
 import json
+from dateutil.parser import isoparse
+from dateutil.tz import gettz, tzutc
 
 console = Console()
 datetime_formatter_one = "%Y-%m-%d %H:%M:%S"
@@ -71,9 +73,55 @@ class gpsLocating(Enum):
     GNSS = 5
 
 
-class Device:
+class severity(Enum):
+    Critical = 0
+    Major = 1
+    Minor = 2
+
+
+class eventStatus(Enum):
+    failed = 0
+    success = 1
+
+
+class ObjBase:
+
+    def __init__(self, raw) -> None:
+        self.raw = raw
+        self.__json = ''
+
+    def utc_to_local(self, dt):
+        if type(dt) is datetime:
+            return dt
+        dt = isoparse(dt)
+        if not dt.tzinfo:
+            return dt.replace(tzinfo=tzutc()).astimezone()
+        return dt.astimezone()
+
+    @property
+    def json(self):
+        self.__json
+
+
+class ObjsBase:
+
+    def __init__(self, objs) -> None:
+        self.objs = objs
+
+    @property
+    def json(self):
+        """Construct every objs' dict format into a list
+
+        Returns:
+            list: List of every objects
+        """
+        return [obj.json for obj in self.objs] if self.objs else []
+
+
+class Device(ObjBase):
 
     def __init__(self, device_content: dict) -> None:
+        self.raw = device_content
         self.__json = device_content
         self.alias = device_content.get('alias', None)
         self.attributes = device_content.get('attributes', None)
@@ -81,8 +129,9 @@ class Device:
         self.certificate = device_content.get('certificate', None)
         self.createTime = device_content.get('createTime', None)
         if self.createTime:
-            self.createTime = datetime.strptime(self.createTime,
-                                                datetime_formatter_one)
+            self.createTime = self.utc_to_local(self.createTime)
+            self.__json['createTime'] = self.createTime.strftime(
+                "%Y-%m-%d %H:%M:%S")
         self.creator = device_content.get('creator', None)
         self.description = device_content.get('description', None)
         self.group = device_content.get('group', None)
@@ -99,28 +148,29 @@ class Device:
         self.sensorOverwrite = device_content.get('sensorOverwriteodel', None)
         self.sensorSn = device_content.get('sensorSn', None)
         self.status = deviceStatus(device_content.get('status', None))
+        if self.status:
+            self.__json['status'] = self.status.name
         self.tags = device_content.get('tags', None)
         self.tempOverwrite = device_content.get('tempOverwrite', None)
         self.template = device_content.get('template', None)
         if self.template is not None and len(self.template) < 3:
             self.template = Template.get_template(self.template['id'])
+            self.__json['template'] = self.template.json
         self.updateTime = device_content.get('updateTime', None)
+        if self.updateTime:
+            self.updateTime = self.utc_to_local(self.updateTime)
+            self.__json['updateTime'] = self.updateTime.strftime(
+                "%Y-%m-%d %H:%M:%S")
         self.updator = device_content.get('updator', None)
         self.latest = device_content.get('latest', None)
         if self.latest:
             self.latest = Report(self.latest)
+            self.__json['latest'] = self.latest.json
 
     @property
     def json(self):
         return self.__json
 
-    def printJson(self):
-        data = self.json
-        data['status'] = self.status.name
-        data['latest']['gpsLocating'] = self.latest.gpsLocating.name
-        data['template'] = self.template.json
-        console.print_json(data=self.json)
-
     def get_latest(self):
         ...
         return self.latest
@@ -146,7 +196,7 @@ class Device:
             self.status.name,
             self.creator,
             str(self.createTime),
-            self.updateTime,
+            str(self.updateTime),
             self.model.name,
             self.group.name,
             self.pid,
@@ -168,7 +218,7 @@ class Devices(list):
         return super().append(__object)
 
 
-class Group:
+class Group(ObjBase):
     ...
 
     def __init__(self, group_dict) -> None:
@@ -179,7 +229,7 @@ class Group:
         return self.name
 
 
-class Model:
+class Model(ObjBase):
     ...
 
     def __init__(self, model_dict) -> None:
@@ -197,11 +247,22 @@ class Company:
         pass
 
 
-class Location:
-    pass
+class Location(ObjBase):
+    ...
+
+    def __init__(self, location_dict) -> None:
+        self.__json = location_dict
+        self.radius = location_dict.get('radius', None)
+        self.latitude = location_dict.get('latitude', None)
+        self.longitude = location_dict.get('longitude', None)
+        self.altitude = location_dict.get('altitude', None)
+
+    @property
+    def json(self):
+        return self.__json
 
 
-class Report:
+class Report(ObjBase):
     ...
 
     def __init__(self, report_dict) -> None:
@@ -210,13 +271,33 @@ class Report:
         self.gpsLocating = report_dict.get('gpsLocating', None)
         if self.gpsLocating is not None:
             self.gpsLocating = gpsLocating(self.gpsLocating)
+            self.__json['gpsLocating'] = self.gpsLocating.name
+        self.location = report_dict.get('location', None)
+        if self.location:
+            self.location = Location(
+                self.location) if type(self.location) is dict else Location(
+                    json.loads(self.location))
+            self.__json['location'] = self.location.json
         self.eid = report_dict.get('eid', None)
         self.ver = report_dict.get('ver', None)
         self.reportLatency = report_dict.get('reportLatency', None)
         self.iccidChange = report_dict.get('iccidChange', None)
         self.pc_vendor = report_dict.get('pc_vendor', None)
         self.prev_report_time = report_dict.get('prev_report_time', None)
+        if self.prev_report_time:
+            self.prev_report_time = self.utc_to_local(self.prev_report_time)
+            self.__json['prev_report_time'] = self.prev_report_time.strftime(
+                "%Y-%m-%d %H:%M:%S")
+        self.prev_info_time = report_dict.get('prev_report_time', None)
+        if self.prev_info_time:
+            self.prev_info_time = self.utc_to_local(self.prev_info_time)
+            self.__json['prev_info_time'] = self.prev_report_time.strftime(
+                "%Y-%m-%d %H:%M:%S")
         self.systemTime = report_dict.get('systemTime', None)
+        if self.systemTime:
+            self.systemTime = self.utc_to_local(self.systemTime)
+            self.__json['systemTime'] = self.systemTime.strftime(
+                "%Y-%m-%d %H:%M:%S")
         self.mfg = report_dict.get('mfg', None)
         self.fw = report_dict.get('fw', None)
         self.iccid = report_dict.get('iccid', None)
@@ -225,9 +306,11 @@ class Report:
         self.timeInterval = report_dict.get('timeInterval', None)
         self.model = report_dict.get('model', None)
         self.time = report_dict.get('time', None)
+        if self.time:
+            self.time = self.utc_to_local(self.time)
+            self.__json['time'] = self.time.strftime("%Y-%m-%d %H:%M:%S")
         self.power = report_dict.get('power', None)
         self.sn = report_dict.get('sn', None)
-        self.prev_info_time = report_dict.get('prev_info_time', None)
         self.ipaddr = report_dict.get('ipaddr', None)
         self.net = report_dict.get('net', None)
         self.cellid = report_dict.get('cellid', None)
@@ -236,7 +319,6 @@ class Report:
         self.mnc = report_dict.get('mnc', None)
         self.rssi = report_dict.get('rssi', None)
         self.battery = report_dict.get('battery', None)
-        self.prev_report_time = report_dict.get('prev_report_time', None)
 
     @property
     def columns(self):
@@ -271,8 +353,9 @@ class Report:
             str(self.power),
             str(self.battery),
             str(self.timeInterval),
-            self.prev_report_time,
-            self.time,
+            self.prev_report_time.strftime("%Y-%m-%d %H:%M:%S")
+            if self.prev_report_time else None,
+            self.time.strftime("%Y-%m-%d %H:%M:%S"),
         ]
         return data
 
@@ -303,6 +386,7 @@ class Report:
         ]
         return columns
 
+    @property
     def json(self):
         return self.__json
 
@@ -312,41 +396,35 @@ class Report:
         return ''
 
 
-class Reports(list):
+class Reports(ObjsBase):
+    ...
+    # @property
+    # def table(self):
+    #     table = Table(show_header=True, header_style="bold magenta")
+    #     for c in self.columns:
+    #         table.add_column(c)
+    #     for row in self.dataRows:
+    #         table.add_row(*row)
+    #     return table
 
-    def __init__(self, reports: List[Report]) -> None:
-        self.__reports = reports
+    # @property
+    # def dataRows(self):
+    #     rows = [report.dataRow for report in self.__reports]
+    #     return rows
 
-    def append(self, __object: Report) -> None:
-        return super().append(__object)
+    # def __repr__(self) -> str:
+    #     console.print(self.table)
 
-    @property
-    def table(self):
-        table = Table(show_header=True, header_style="bold magenta")
-        for c in self.columns:
-            table.add_column(c)
-        for row in self.dataRows:
-            table.add_row(*row)
-        return table
-
-    @property
-    def dataRows(self):
-        rows = [report.dataRow for report in self.__reports]
-        return rows
-
-    def __repr__(self) -> str:
-        console.print(self.table)
-
-        return ''
+    #     return ''
 
 
-class Sensor:
+class Sensor(ObjBase):
 
     def __init__(self) -> None:
         pass
 
 
-class Template:
+class Template(ObjBase):
 
     base_url = TEMPLATE_URL
 
@@ -362,6 +440,8 @@ class Template:
         self.description = template_dict.get('description', None)
         self.creator = template_dict.get('creator', None)
         self.createTime = template_dict.get('createTime', None)
+        if self.createTime:
+            self.createTime = self.utc_to_local(self.createTime)
         self.updator = template_dict.get('updator', None)
         self.updatetime = template_dict.get('updatetime', None)
         self.model = template_dict.get('model', None)
@@ -407,28 +487,102 @@ class Template:
 
     @property
     def json(self):
+        if self.__json.get('configure', None):
+            self.__json['configure'] = self.configure.json
         return self.__json
 
-    #TODO make configures class
-
     class Configure(object):
         ...
+        json = {}
 
         def __init__(self, configure) -> None:
             if configure:
                 for sensor in configure['settings']:
                     setattr(self, sensor['sensorName'], Sensor())
                     for rule in sensor['ruleset']:
+                        self.json[rule[
+                            'ruleName']] = rule['ruleValue'] + ' ' + rule.get(
+                                'unit', '')
                         setattr(getattr(self, sensor['sensorName']),
                                 rule['ruleName'],
                                 rule['ruleValue'] + ' ' + rule.get('unit', ''))
 
+        def __repr__(self) -> str:
+            return self.json
 
-class Event:
+
+class Event(ObjBase):
     ...
 
+    def __init__(self, event_dict) -> None:
+        self.raw = event_dict
+        self.__json = event_dict
+        self.sn = event_dict.get('sn', None)
+        self.status = eventStatus(event_dict.get('status', None))
+        if self.status is not None:
+            self.__json['status'] = self.status.name
+        self.severity = severity(event_dict.get('severity', None))
+        if self.severity is not None:
+            self.__json['severity'] = self.severity.name
+        self.origin = event_dict.get('origin', None)
+        self.code = event_dict.get('code', None)
+        self.category = event_dict.get('category', None)
+        self.additionInfo = event_dict.get('additionInfo', None)
+        if self.additionInfo:
+            self.additionInfo = json.loads(self.additionInfo)
+            self.__json['additionInfo'] = self.additionInfo
+        self.time = event_dict.get('time', None)
+        if self.time:
+            self.time = self.utc_to_local(self.time)
+            self.__json['time'] = self.time.strftime("%Y-%m-%d %H:%M:%S")
 
-class Events(list):
+    @property
+    def table(self):
+        table = Table(show_header=True, header_style="bold magenta")
+        for c in self.columns:
+            table.add_column(c)
+        table.add_row(*self.dataRow)
+        return table
+
+    @property
+    def columns(self):
+        columns = [
+            'sn',
+            'status',
+            'severity',
+            'origin',
+            'code',
+            'category',
+            'additionInfo',
+            'time',
+        ]
+        return columns
+
+    @property
+    def dataRow(self):
+        data = [
+            self.sn,
+            str(self.status),
+            str(self.severity),
+            self.origin,
+            self.code,
+            self.category,
+            json.dumps(self.additionInfo) if self.additionInfo else None,
+            self.time.strftime("%Y-%m-%d %H:%M:%S"),
+        ]
+        return data
+
+    @property
+    def json(self):
+        return self.__json
+
+    def __repr__(self) -> str:
+        ...
+        console.print(self.table)
+        return ''
+
+
+class Events(ObjsBase):
     ...
 
 
@@ -458,7 +612,7 @@ class Pages:
 
     def collect_contents(self, useClass, params: dict = None):
         ...
-
+        if not self.content: return None, None
         if len(self.content) < self.totalElements:
             # server max get size = 2000
             # if oversize then need to query by page
